import os
import errno
import configparser
import click
from ..constants import APP_NAME


@click.command()
@click.option('--subdomain', type=click.STRING, prompt=True)
@click.option('--email', type=click.STRING, prompt=True)
@click.option('--password', type=click.STRING, prompt=True, hide_input=True)
@click.pass_context
def configure(ctx, subdomain, email, password):
    """Configure Zendesk authentication."""
    profile = ctx.obj['profile']

    conf_path = os.path.join(click.get_app_dir(APP_NAME), 'config.ini')
    config = configparser.ConfigParser()
    config.read([conf_path])

    try:
        config.add_section(profile)
    except configparser.DuplicateSectionError:
        click.confirm('That profile already exists. Overwrite?', abort=True)

    config.set(profile, 'Subdomain', subdomain)
    config.set(profile, 'Email', email)
    config.set(profile, 'Password', password)

    try:
        os.makedirs(click.get_app_dir(APP_NAME))
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

    with open(conf_path, 'w') as configfile:
        config.write(configfile)
        click.echo('Config written to %s' % conf_path)



import click
import simplejson as json
from ..utilities import get_all_macros


@click.command()
@click.option('--directory', type=click.Path(exists=True, file_okay=False, writable=True, resolve_path=True),
              default='.')
@click.option('--filename', type=click.STRING, default='macros.json')
@click.pass_context
def get_macros(ctx, directory, filename):
    """Get all macros and save to file."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    macros = get_all_macros(config=ctx.obj['configuration'])
    path = '%s/%s' % (directory, filename)

    with open(path, 'w') as outfile:
        json.dump({'macros': macros}, outfile, indent=2)

    click.echo('Macros saved to %s' % path)



import os
from time import time
import click
from ..utilities import get_all_locales, get_all_hc_by_type, write_json


@click.command()
@click.option('--directory', type=click.Path(exists=True, file_okay=False, writable=True, resolve_path=True),
              default='.')
@click.pass_context
def backup_guide(ctx, directory):
    """Backup Guide categories, sections and articles."""
    if ctx.obj['configuration'] == {}:
        raise click.UsageError('No configuration found. Try `zenkly configure`', ctx=ctx)

    # Get all of the available locales within Zendesk
    locale_data = get_all_locales(config=ctx.obj['configuration'])
    locales = [l['locale'].lower() for l in locale_data]

    backup_time = int(time())

    for locale in locales:
        for type in ['articles', 'sections', 'categories']:
            data = get_all_hc_by_type(config=ctx.obj['configuration'], locale=locale, type=type)

            output_path = os.path.join(directory, 'backup_%s' % backup_time, locale)
            filename = '%s.json' % type

            write_json(output_path=output_path, filename=filename, data=data)



from setuptools import setup, find_packages


setup(
    name='zenkly',
    version='0.1.12',
    packages=find_packages(),
    description='CLI for Zendesk admin tasks',
    author='Zena Hirsch',
    author_email='hirsch.zena@gmail.com',
    url='https://github.com/zenahirsch/zenkly',
    include_package_data=True,
    package_data={'zenkly': ['schemas/*.schema']},
    install_requires=[
        'Click',
        'simplejson',
        'requests',
        'jsonschema',
        'colorama',
        'configparser',
    ],
    entry_points='''
        [console_scripts]
        zenkly=zenkly.scripts.zenkly:cli
    ''',
)


import os
import errno
import csv
import json
import click
import requests
import time
import threading
from functools import wraps


def rate_limited(max_per_second: int):
    """
    Rate-limits the decorated function locally, for one process.
    :param max_per_second: the number of requests allowed per second
    :return: the decorated function
    """
    lock = threading.Lock()
    min_interval = 1.0 / max_per_second

    def decorate(func):
        last_time_called = time.perf_counter()

        @wraps(func)
        def rate_limited_function(*args, **kwargs):
            lock.acquire()
            nonlocal last_time_called
            try:
                elapsed = time.perf_counter() - last_time_called
                left_to_wait = min_interval - elapsed
                if left_to_wait > 0:
                    time.sleep(left_to_wait)

                return func(*args, **kwargs)
            finally:
                last_time_called = time.perf_counter()
                lock.release()

        return rate_limited_function

    return decorate


@rate_limited(1)
def get(config, url):
    """
    GET the provided endpoint.
    :param config: context config
    :param url: the url to GET
    :return:
    """
    r = requests.get(
        url,
        auth=(config['email'], config['password'])
    )

    try:
        res = r.json()
        if 'error' in res:
            raise click.UsageError(res['error'])
    except ValueError:
        res = r.text

    if 'error' in res:
        raise click.UsageError(res['error'])

    return res


@rate_limited(1)
def put(config, url, data):
    """
    PUT data to the provided endpoint.
    :param config: context config
    :param url: the url to PUT data to
    :param data: the data to PUT
    :return:
    """
    r = requests.put(
        url,
        auth=(config['email'], config['password']),
        json=data
    )

    try:
        res = r.json()
        if 'error' in res:
            raise click.UsageError(res['error'])
    except ValueError:
        res = r.text

    return res


@rate_limited(1)
def post(config, url, data):
    """
    POST data to the provided endpoint.
    :param config: context config
    :param url: the url to POST the data to
    :param data: the data to POST
    :return:
    """
    r = requests.post(
        url,
        auth=(config['email'], config['password']),
        json=data
    )

    try:
        res = r.json()
        if 'error' in res:
            raise click.UsageError(res['error'])
    except ValueError:
        res = r.text

    return res


def get_all_macros(config):
    """
    Get all pages of macros from Zendesk.
    :param config: context config
    :return: list of macro objects
    """
    url = 'https://%s.zendesk.com/api/v2/macros.json' % config['subdomain']
    res = get(config, url)
    all = res['macros']

    with click.progressbar(length=res['count'], label='Getting macros...') as bar:
        bar.update(len(res['macros']))

        while res['next_page']:
            res = get(config, res['next_page'])
            all = all + res['macros']
            bar.update(len(res['macros']))

    return all


def post_all_macros(config, data):
    """
    Create macros in Zendesk.
    :param config: context config
    :param data: the macro data to POST
    """
    entries = (
        'title', 'active', 'actions', 'restriction',
        'description', 'attachments'
    )

    succeeded = []
    failed = []

    with click.progressbar(length=len(data), label='Adding macros...') as bar:
        for m in data:
            macro = {'macro': {k: m[k] for k in m if k in entries}}

            url = 'https://%s.zendesk.com/api/v2/macros.json' % config['subdomain']

            try:
                res = post(config, url, macro)
                succeeded.append((m['id'], res['macro']['id']))
            except click.UsageError as e:
                failed.append((m['id'], e.message))

            bar.update(1)

        click.secho('\n\nAddition complete!')

        if succeeded:
            click.secho('\nThe following macros were added: ', fg='green', bold=True)
            for s in succeeded:
                click.secho('%d (new id: %d)' % (s[0], s[1]), fg='green')

        if failed:
            click.secho('\nThe following macros could not be added: ', fg='red', bold=True)
            for f in failed:
                click.secho('%d (%s)' % (f[0], f[1]), fg='red')


def put_all_macros(config, data):
    """
    Update macros in Zendesk.
    :param config: context config
    :param data: the macro data to PUT
    """
    entries = (
        'title', 'active', 'actions', 'restriction',
        'description', 'attachments'
    )

    succeeded = []
    failed = []

    with click.progressbar(length=len(data), label='Updating macros...') as bar:
        for m in data:
            macro = {'macro': {k: m[k] for k in m if k in entries}}

            url = 'https://%s.zendesk.com/api/v2/macros/%d.json' % (config['subdomain'], m['id'])

            try:
                put(config, url, macro)
                succeeded.append(m['id'])
            except click.UsageError as e:
                failed.append((m['id'], e.message))

            bar.update(1)

        click.secho('\n\nUpdate complete!')

        if succeeded:
            click.secho('\nThe following macros were updated: ', fg='green', bold=True)
            for s in succeeded:
                click.secho('%d' % s, fg='green')

        if failed:
            click.secho('\nThe following macros could not be updated: ', fg='red', bold=True)
            for f in failed:
                click.secho('%d (%s)' % (f[0], f[1]), fg='red')


def get_all_locales(config):
    url = 'https://%s.zendesk.com/api/v2/locales.json' % config['subdomain']
    res = get(config, url)
    all_locales = res['locales']

    with click.progressbar(length=res['count'], label='Getting locales...') as bar:
        bar.update(len(res['locales']))

        while res['next_page']:
            res = get(config, res['next_page'])
            all_locales.append(res['locales'])
            bar.update(len(res['locales']))

    return all_locales


def get_all_hc_by_type(config, locale, type):
    url = 'https://%s.zendesk.com/api/v2/help_center/%s/%s.json' % (config['subdomain'], locale, type)
    res = get(config, url)
    all_data = res[type]

    with click.progressbar(length=res['count'], label='Getting %s for locale %s...' % (type, locale)) as bar:
        bar.update(len(res[type]))

        while res['next_page']:
            res = get(config, res['next_page'])
            all_data.append(res[type])
            bar.update(len(res[type]))

    return all_data


def confirm_or_create_path(path):
    # check if path exists, and create it if it doesn't
    if not os.path.exists(path):
        try:
            os.makedirs(path)
        except OSError as error:
            if error.errno != errno.EEXIST:
                raise


def write_json(output_path, filename, data):
    confirm_or_create_path(output_path)

    with open(os.path.join(output_path, filename), 'w') as outfile:
        json.dump(data, outfile, indent=4)
